//
//  FeedListViewModel.swift
//  ReviewsFeed
//
//  Created by Javier Cicchelli on 18/03/2024.
//  Copyright © 2024 Röck+Cöde. All rights reserved.
//

import Foundation
import ReviewsiTunesKit

extension FeedListViewController {
    final class ViewModel: ObservableObject {
        
        // MARK: Type aliases
        typealias Configuration = FeedListViewController.Configuration
        
        // MARK: Constants
        private let configuration: Configuration

        // MARK: Properties
        @Published var filter: FilterOption = .all
        @Published var isFilterEnabled: Bool = false
        @Published var isLoading: Bool = false

        var items: [Review] = []
        
        lazy private var iTunesService: iTunesService = {
            .init(configuration: .init(session: configuration.session))
        }()

        // MARK: Initialisers
        init(configuration: Configuration = .init()) {
            self.configuration = configuration
        }

        // MARK: Functions
        func fetch() {
            Task {
                isFilterEnabled = false
                isLoading = items.isEmpty

                do {
                    let output = try await iTunesService.getReviews(.init(
                        appID: configuration.appID,
                        countryCode: configuration.countryCode
                    ))
                    
                    items = output.reviews
                        .map { review -> Review in
                            .init(
                                author: review.author,
                                comment: review.content,
                                id: review.id,
                                rating: .init(
                                    stars: review.rating,
                                    appVersion: review.version
                                ),
                                title: review.title
                            )
                        }
                    isFilterEnabled = !items.isEmpty
                } catch {
                    // TODO: handle this error gracefully.
                    print("ERROR: \(error.localizedDescription)")
                }

                isLoading = false
            }
        }

    }
}
